package img2pdf;

import com.foxit.gsdk.PDFException;
import com.foxit.gsdk.image.Image;
import com.foxit.gsdk.pdf.PDFDocument;
import com.foxit.gsdk.pdf.PDFPage;
import com.foxit.gsdk.pdf.pageobjects.ImageObject;
import com.foxit.gsdk.pdf.pageobjects.PageObject;
import com.foxit.gsdk.pdf.pageobjects.PageObjects;
import com.foxit.gsdk.utils.FileHandler;
import com.foxit.gsdk.utils.Matrix;
import com.foxit.gsdk.utils.Size;
import common.Common;

public class img2pdf
{
	// load Foxit PDF Library
	static
	   {
	      try
	      {
	        String name = System.getProperty("os.name");
	        String arch = System.getProperty("os.arch");
	        System.out.println("os.name = " + name);
	        System.out.println("os.arch = " + arch);
	           if (name.toLowerCase().startsWith("win"))//windows
	        {
	             if (arch.contains("64"))
	           {
	                System.load(System.getProperty("user.dir") + "//lib//fsdk_java_win64.dll");
	           }
	             else {
	                System.load(System.getProperty("user.dir") + "//lib//fsdk_java_win32.dll");
	           }
	        }
	           else {//linux
	           if (arch.contains("64"))
	              System.load(System.getProperty("user.dir") + "/lib/libfsdk_java_linux64.so");
	           else {
	              System.load(System.getProperty("user.dir") + "/lib/libfsdk_java_linux32.so");
	           }
	        }

	      }
	      catch (UnsatisfiedLinkError e)
	      {
	        System.out.println("Native code library failed to load.\n" + e);
	        System.exit(1);
	      }
	   }

	
	//The default page size
	private final static float pageWidth = 612;
	private final static float pageHeight = 792;
	private final static float margin = 10;
	
	public static void main(String[] args)
	{
		Common common = new Common();
		
		String inputSrcFile = "";
		String outputFloder = "";
		
		//Judge if to command line arguments. 
		//usage:java img2pdf <srcfile> <destfloder>
		if (args.length > 1)
		{
			inputSrcFile = args[0];
			if (args.length >= 2)
			{
				outputFloder = args[1];
			}
			
			if (!Common.fileExist(inputSrcFile) || (outputFloder != "" && !Common.folderExist(outputFloder)))
			{
				System.out.println("Failed: The input source file or output floder is not exist!");
				System.exit(1);
			}
		}
		
		if (outputFloder == "")
		{
			outputFloder = "../output_files/img2pdf";
			Common.createFloder(outputFloder);
		}
		
		//open and write
		Common.openLog(outputFloder + "/log.txt");
		Common.outputLog("Foxit PDF Linux JAVA SDK example: img2pdf");
		
		common.initlize();
		
		if (inputSrcFile != "")
		{
			imageToPDF(inputSrcFile, outputFloder);
		}
		else {
			//use default path
			inputSrcFile = "../testfiles/";
			imageToPDF(inputSrcFile + "1.jpg", outputFloder);
			imageToPDF(inputSrcFile + "2.jpg", outputFloder);
			imageToPDF(inputSrcFile + "3.jpg", outputFloder);
			imageToPDF(inputSrcFile + "TIF2Pages.tif", outputFloder);
		}
				
		//release the resource		
		common.release();
		//close log file
		Common.closeLog();
	}
	
	public static void imageToPDF(String inputSrcFile, String outputFloder)
	{
		try
		{
			FileHandler imageFile = FileHandler.create(inputSrcFile, FileHandler.FILEMODE_READONLY);
			Image image = Image.load(imageFile);
			Common.outputLog("Success: Load image from image file.");
			int nFrameCount = 0;
			nFrameCount = image.countFrames();
			Common.outputLog("Success: count image frames. the count is " + nFrameCount);
			PDFDocument pdfDocument = Common.createDocument();
			for (int i = 0; i < nFrameCount; i++)
			{
				//create page
				PDFPage page = Common.createPage(pdfDocument, i);
				//set page size
				page.setSize(pageWidth, pageHeight);
				// Insert some frame of the image into page
				insertImageObject(image, page, i);
			}
			
			//save the document as a PDF file
			String imageName = Common.getFileName(inputSrcFile);
			String fileName = imageName.substring(0, imageName.length() - 4) + ".pdf";
			Common.saveDocument(pdfDocument, outputFloder + "/" + fileName);
			
			//release
			imageFile.release();
			image.release();
			pdfDocument.close();
		}
		catch (PDFException e)
		{
			e.printStackTrace();
			Common.outputErrMsg(e.getLastError(), "Failed to tansform image to PDF document.");
		}	
	}

	public static void insertImageObject(Image image, PDFPage page, int frameIndex)
	{
		// Load an image based on frame index
		ImageObject imageObject = null;
		try
		{
			image.loadFrame(frameIndex);
			imageObject = ImageObject.create(page);
			Common.outputLog("Success: Create an image object with the specified page.");
			imageObject.setImage(image);

			Size size = image.getCurrentFrameSize();
			int width = size.getWidth();
			int height = size.getHeight();
			// the ratio of image width to page width
			float w = (float) width/pageWidth;
			// the ratio of image height to page height
			float h = (float) height/pageHeight;
			
			float scale = 1;
			float left = 0;
			float bottom = 0;

			// Calculate the position and scale of image object
			// which should be placed in a PDF
			if ( w > h ){		 
				if (w > 1.0){
					// Horizontal will be scaled to have the same width with page
					scale = 1/w;
					width = (int) (width * scale - 2 * margin);
					height = (int) (height * scale);

					left = margin;
					bottom = (pageHeight - height)/2;
				}
				else{
					// Both horizontal and vertical don't need to be scaled.
					left = (pageWidth - width)/2;
					bottom = (pageHeight - height)/2;
				}
			}
			else{
				// Vertical will be scaled to have the same height with page
				if( h> 1.0){
					scale = 1/h;
					width = (int) (width * scale);
					height = (int) (height * scale - 2 * margin);
					left = (pageWidth - width)/2;
					bottom = margin;
				}
				else{
					// Both horizontal and vertical don't need to be scaled.
					left = (pageWidth - width)/2;
					bottom = (pageHeight - height)/2;
				}
			}
			
			// Compose an image matrix based on calculation as above
			Matrix imageMatrix = new Matrix();
			float[] values = new float[] {(float)width, 0, left, 0, (float)height, bottom, 0, 0, 0};
			imageMatrix.setValues(values);			
			imageObject.setMatrix(imageMatrix);

			// Get page objects in a page and prepare to insert an image object
			PageObjects pageObjects = page.getPageObjects();

			// Insert an image object into page objects which in a page 
			pageObjects.insertObject(PageObject.TYPE_ALL, 1, imageObject);
			Common.outputLog("Success: Insert an image object into page objects");
			// Generate contents which are covered in page objects
			pageObjects.generateContents();			
		}
		catch (PDFException e)
		{
			e.printStackTrace();
			if (imageObject != null)
			{
				try
				{
					imageObject.release();
				}
				catch (PDFException e1)
				{
					e1.printStackTrace();
				}
			}
			Common.outputErrMsg(e.getLastError(), "Failed to insert an image to PDF document.");
		}
	}
}
